@@ -7,7 +7,6 @@ import pysam
 import sys
 import re
 import numpy as np
-import seaborn as sns
 import matplotlib
 import pandas as pd
 import copy
@@ -21,6 +20,7 @@ from ingenannot.utils.gff_reader import GFF3Reader, GTFReader
 from ingenannot.utils.gene_builder import GeneBuilder
 from ingenannot.utils.annot_edit_distance import AnnotEditDistance
 from ingenannot.commands.command import Command
+from ingenannot.utils.graphics import Graphics
 
 class Select(Command):
 
@@ -118,7 +118,7 @@ class Select(Command):
         for mg in metagenes:
             current_mg_export_tr = []
             current_mg_not_export_tr = []
-            lsorted_tmp = self._rank_tr(mg.lTranscripts)
+            lsorted_tmp = Utils.rank_transcripts(mg.lTranscripts, self.gaeval)
 
 
             # limit potential transcripts to coordinates (use for rescue overlapping CDS)
@@ -275,193 +275,6 @@ class Select(Command):
         f.close()
 
 
-
-#    @staticmethod
-    def _rank_tr(self, transcripts):
-        """double sort tr and pr, compare list, when
-           discrepency, use distances"""
-
-        l_sorted = []
-        l_tr_rank = sorted(transcripts, key=lambda tr: (min(tr.best_tr_evidence[1],tr.best_lg_evidence[1]), tr.best_bx_evidence[1]))
-        l_pr_rank = sorted(transcripts, key=lambda tr: (tr.best_bx_evidence[1], min(tr.best_tr_evidence[1],tr.best_lg_evidence[1])))
-        for idx in range(0,len(transcripts)):
-            tr_to_remove = None
-            if l_tr_rank[0] == l_pr_rank[0]:
-                tr_to_remove = l_tr_rank[0]
-            elif l_tr_rank[0].best_bx_evidence[1] == l_pr_rank[0].best_bx_evidence[1]:
-                tr_to_remove = l_tr_rank[0]
-            else:
-                if self.gaeval:
-                    if l_pr_rank[0].gaeval_infos['int'] > l_tr_rank[0].gaeval_infos['int']:
-                        tr_to_remove = l_pr_rank[0]
-                    else:
-                        tr_to_remove = l_tr_rank[0]
-                else:
-                    tr_delta = l_pr_rank[0].best_tr_evidence[1] - l_tr_rank[0].best_tr_evidence[1]
-                    # add stronger weigth on protein distance 
-#                    pr_delta = (l_tr_rank[0].best_bx_evidence[1] - l_pr_rank[0].best_bx_evidence[1]) * 1.5
-                    pr_delta = l_tr_rank[0].best_bx_evidence[1] - l_pr_rank[0].best_bx_evidence[1]
-
-                    # To validate BUG ? order
-                    if tr_delta >= pr_delta:
-                        tr_to_remove = l_tr_rank[0]
-                    else:
-                        tr_to_remove = l_pr_rank[0]
-            l_sorted.append(tr_to_remove)
-            l_tr_rank.remove(tr_to_remove)
-            l_pr_rank.remove(tr_to_remove)
-
-        return l_sorted
-
-
-
-    def get_aed(self, genes):
-
-        logging.info("Retrieving aed annotations")
-
-        for g in genes:
-            for tr in g.lTranscripts:
-                if "aed_ev_tr" in tr.dAttributes and "ev_tr" in tr.dAttributes:
-                    tr.best_tr_evidence = (tr.dAttributes["ev_tr"][0],float(tr.dAttributes["aed_ev_tr"][0]))
-                else:
-                    logging.error("Problem: transcript [{}] of gene [{}], source [{}], missing aed_tr annotations".format(tr.id, g.gene_id, g.source))
-                if "aed_ev_pr" in tr.dAttributes and "ev_pr" in tr.dAttributes:
-                    tr.best_bx_evidence = (tr.dAttributes["ev_pr"][0],float(tr.dAttributes["aed_ev_pr"][0]))
-                else:
-                    logging.error("Problem: transcript [{}] of gene [{}], source [{}], missing aed_pr annotations".format(tr.id, g.gene_id, g.source))
-                if "aed_ev_lg" in tr.dAttributes and "ev_lg" in tr.dAttributes:
-                    tr.best_lg_evidence = (tr.dAttributes["ev_lg"][0],float(tr.dAttributes["aed_ev_lg"][0]))
-
-                if "ev_lg_penalty" in tr.dAttributes:
-                    tr.lg_penalty = tr.dAttributes["ev_lg_penalty"][0]
-                if "ev_tr_penalty" in tr.dAttributes:
-                    tr.tr_penalty = tr.dAttributes["ev_tr_penalty"][0]
-
-    def get_values_for_scatter_hist(self, transcripts):
-
-        l_aed_tr = []
-        l_aed_tr_no_penalty = []
-        l_aed_pr = []
-        l_aed_pr_no_penalty = []
-
-        for tr in transcripts:
-            l_aed_pr.append(tr.best_bx_evidence[1])
-            if self.use_ev_lg:
-                l_aed_tr.append(min(tr.best_tr_evidence[1],tr.best_lg_evidence[1]))
-            else:
-                l_aed_tr.append(tr.best_tr_evidence[1])
-
-            if tr.tr_penalty != 'yes' and tr.lg_penalty != 'yes':
-                l_aed_pr_no_penalty.append(tr.best_bx_evidence[1])
-                if self.use_ev_lg:
-                    l_aed_tr_no_penalty.append(min(tr.best_tr_evidence[1],tr.best_lg_evidence[1]))
-                else:
-                    l_aed_tr_no_penalty.append(tr.best_tr_evidence[1])
-
-
-        return l_aed_tr, l_aed_tr_no_penalty, l_aed_pr, l_aed_pr_no_penalty
-
-
-
-    def scatter_hist(self, laed, out="", legend="", title=""):
-        """scatter plot of AEDs with histograms"""
-
-        plt.style.use('bmh')
-        #plt.style.use('seaborn')
-        #plt.style.use('ggplot')
-
-        fig = plt.Figure(figsize=(20,20))
-        # add gridspec
-        gs = fig.add_gridspec(2,2,width_ratios=(7, 2), height_ratios=(2, 7),
-                                      left=0.1, right=0.9, bottom=0.1, top=0.9,
-                                                            wspace=0.05, hspace=0.05)
-        ax = fig.add_subplot(gs[1, 0])
-        ax_histx = fig.add_subplot(gs[0, 0], sharex=ax)
-        ax_histy = fig.add_subplot(gs[1, 1], sharey=ax)
-        ax_legend = fig.add_subplot(gs[0, 1])
-
-        ax.scatter(laed[0],laed[1], color="#20C2EF")
-        # By using ``transform=vax.get_xaxis_transform()`` the y coordinates are scaled
-        # such that 0 maps to the bottom of the axes and 1 to the top.
-        ax.vlines(self.aedtr_filtering, 0, 1, transform=ax.get_xaxis_transform(), colors='r', linestyle="dashed")
-        ax.text(self.aedtr_filtering,0.9,self.aedtr_filtering,size=20,ha="center", va="center", color='r',bbox=dict(boxstyle="round",fc='#EEEEEE'))
-        ax.hlines(self.aedpr_filtering, 0, 1, transform=ax.get_yaxis_transform(), colors='r', linestyle="dashed")
-        ax.text(0.9,self.aedpr_filtering,self.aedpr_filtering,size=20,ha="center", va="center", color='r', bbox=dict(boxstyle="round",fc='#EEEEEE'))
-
-        lb,rb,lt,rt = 0,0,0,0
-        lb_no_penalty,rb_no_penalty,lt_no_penalty,rt_no_penalty = 0,0,0,0
-
-        for i,val in enumerate(laed[0]):
-            if laed[0][i] <= self.aedtr_filtering and laed[1][i] <= self.aedpr_filtering:
-                lb += 1
-            if laed[0][i] > self.aedtr_filtering and laed[1][i] <= self.aedpr_filtering:
-                rb += 1
-            if laed[0][i] <= self.aedtr_filtering and laed[1][i] > self.aedpr_filtering:
-                lt += 1
-            if laed[0][i] > self.aedtr_filtering and laed[1][i] > self.aedpr_filtering:
-                rt += 1
-        ax.text(self.aedtr_filtering/2,self.aedpr_filtering/2,lb,size=20,ha="center", va="center", bbox=dict(boxstyle="round",ec=(0,0,0), fc=(1., 1., 1.)))
-        ax.text(1-((1-self.aedtr_filtering)/2),self.aedpr_filtering/2,rb,size=20,ha="center", va="center", bbox=dict(boxstyle="round",ec=(0,0,0), fc=(1., 1., 1.)))
-        ax.text(self.aedtr_filtering/2,1-((1-self.aedpr_filtering)/2),lt,size=20,ha="center", va="center", bbox=dict(boxstyle="round",ec=(0,0,0), fc=(1., 1., 1.)))
-        ax.text(1-((1-self.aedtr_filtering)/2),1-((1-self.aedpr_filtering)/2),rt,size=20,ha="center", va="center", bbox=dict(boxstyle="round",ec=(0,0,0), fc=(1., 1., 1.)))
-
-        for i,val in enumerate(laed[2]):
-            if laed[2][i] <= self.aedtr_filtering and laed[3][i] <= self.aedpr_filtering:
-                lb_no_penalty += 1
-            if laed[2][i] > self.aedtr_filtering and laed[3][i] <= self.aedpr_filtering:
-                rb_no_penalty += 1
-            if laed[2][i] <= self.aedtr_filtering and laed[3][i] > self.aedpr_filtering:
-                lt_no_penalty += 1
-            if laed[2][i] > self.aedtr_filtering and laed[3][i] > self.aedpr_filtering:
-                rt_no_penalty += 1
-
-            ax.text(self.aedtr_filtering/2,self.aedpr_filtering/2-0.05,lb_no_penalty,size=20,ha="center", va="center", bbox=dict(boxstyle="round",ec=(0,0,0), fc=(.5, 1., 1.)))
-            ax.text(1-((1-self.aedtr_filtering)/2),self.aedpr_filtering/2-0.05,rb_no_penalty,size=20,ha="center", va="center", bbox=dict(boxstyle="round",ec=(0,0,0), fc=(.5, 1., 1.)))
-            ax.text(self.aedtr_filtering/2,1-((1-self.aedpr_filtering)/2)-0.05,lt_no_penalty,size=20,ha="center", va="center", bbox=dict(boxstyle="round",ec=(0,0,0), fc=(.5, 1., 1.)))
-            ax.text(1-((1-self.aedtr_filtering)/2),1-((1-self.aedpr_filtering)/2)-0.05,rt_no_penalty,size=20,ha="center", va="center", bbox=dict(boxstyle="round",ec=(0.0,0,0), fc=(.5, 1., 1.)))
-
-        ax.set_xlabel("AED with transcript evidence", fontsize=20)
-        ax.set_ylabel("AED with protein evidence", fontsize=20)
-        ax.tick_params(labelsize=15)
-
-        #ax.margins(0.02)
-        #ax.set_xlim((0,1))
-        #ax.set_ylim((0,1))
-
-        bins = np.arange(0.0,1.01,0.01)
-        ax_histx.hist(laed[0], bins=bins, color = '#36953a', edgecolor = 'black', label="AED transcripts")
-        ax_histx.vlines(self.aedtr_filtering, 0, 1, transform=ax_histx.get_xaxis_transform(), colors='r', linestyle="dashed")
-        ax_histx.tick_params(labelsize=12)
-        ax_histx.set_ylabel("Nb. Transcripts", fontsize=20)
-        #ax_histx.legend(fontsize=20)
-        ax_histy.hist(laed[1], bins=bins, color = '#fc4b67', edgecolor = 'black', orientation='horizontal', label="AED proteins")
-        ax_histy.hlines(self.aedpr_filtering, 0, 1, transform=ax_histy.get_yaxis_transform(), colors='r', linestyle="dashed")
-        ax_histy.tick_params(labelsize=12)
-        ax_histy.set_xlabel("Nb. Transcripts", fontsize=20)
-        #ax_histy.legend(fontsize=20)
-
-        h,l=ax_histx.get_legend_handles_labels() # get labels and handles from histx  
-        hy,ly=ax_histy.get_legend_handles_labels() # get labels and handles from histy
-        h.extend(hy)
-        l.extend(ly)
-        ax_legend.legend(h,l, fontsize=20)
-        # Hide grid lines
-        ax_legend.grid(False)
-        # Hide axes ticks
-        #ax_legend.set_xticks([])
-        #ax_legend.set_yticks([])
-        # change background color
-        ax_legend.set_facecolor('w')
-        ax_legend.axis('off')
-
-        canvas = FigureCanvasAgg(fig)
-        canvas.print_figure(out, dpi=80)
-
-        logging.info("Scatter plot exported in {}".format(out))
-
-
-
-
     def run(self):
         """"launch command"""
 
@@ -476,7 +289,7 @@ class Select(Command):
         genes = Utils.extract_genes_from_fof(self.fof)
 
         if self.noaed:
-            self.get_aed(genes)
+            Utils.get_aed_from_attributes(genes)
         if self.gaeval:
             gaeval_infos = Utils.extract_gaeval_from_fof(self.fof)
             Utils.add_gaeval_infos_to_transcripts(genes, gaeval_infos)
@@ -491,14 +304,10 @@ class Select(Command):
         nb_removed_metagenes = nb_metagenes - len(metagenes)
 
         if not self.noaed:
-#           metagenes = self.compute_aed_with_transcripts(metagenes, procs=Command.NB_CPUS)
             metagenes = AnnotEditDistance.compute_aed(metagenes, self.transcript_gff_file, self.transcript_gff_file_stranded, self.transcript_gff_file_source, self.penalty_overflow, evtype="tr", cds_only=self.aed_tr_cds_only, procs=Command.NB_CPUS)
-#            metagenes = self.compute_aed_with_proteins(metagenes, procs=Command.NB_CPUS)
             metagenes = AnnotEditDistance.compute_aed(metagenes, self.protein_gff_file, self.protein_gff_file_stranded, self.protein_gff_file_source, 0.0, evtype="pr", procs=Command.NB_CPUS)
 
             if self.longread_gff_file:
-#                metagenes = self.compute_aed_with_longread_transcripts(metagenes,procs=Command.NB_CPUS)
-
                 metagenes = AnnotEditDistance.compute_aed(metagenes, self.longread_gff_file, True, self.longread_gff_file_source, self.longread_penalty_overflow, evtype="lg", cds_only=self.aed_tr_cds_only, procs=Command.NB_CPUS)
 
         transcripts, transcripts_not_exported = self.filter(metagenes, nb_removed_metagenes)
@@ -523,10 +332,12 @@ class Select(Command):
             logging.info("{} transcripts rescued after overlapping with other CDS".format(nb_rescue))
 
 
-        l_aed_tr, l_aed_tr_no_penalty, l_aed_pr, l_aed_pr_no_penalty = self.get_values_for_scatter_hist(transcripts)
+        l_aed_tr, l_aed_tr_no_penalty, l_aed_pr, l_aed_pr_no_penalty = Graphics.get_values_for_aed_scatter_hist(transcripts,self.use_ev_lg)
 
 
-        self.scatter_hist([l_aed_tr, l_aed_pr, l_aed_tr_no_penalty, l_aed_pr_no_penalty], "{}.scatter_hist_aed.png".format(self.output),legend=['aed_tr','aed_pr'], title="all runs - density of aed")
+        graph_output = "{}.scatter_hist_aed.png".format(self.output)
+        Graphics.plot_aed_scatter_hist([l_aed_tr, l_aed_pr, l_aed_tr_no_penalty, l_aed_pr_no_penalty], self.aedtr_filtering, self.aedpr_filtering, graph_output ,legend=['aed_tr','aed_pr'], title="all runs - density of aed")
+        logging.info("Scatter plot exported in {}".format(graph_output))
         
         self.export(genes,transcripts,self.output)
         if self.no_export :
@@ -537,40 +348,15 @@ class Select(Command):
 
     def aed_strand_filter(self, transcripts):
 
-        conflict_list = []
         transcripts_not_exported = []
 
-        tr_dict = {}
-        references = set([tr.seqid for tr in transcripts])
-        for ref in references:
-            ref_tr = sorted([tr for tr in transcripts if tr.seqid == ref], key=lambda x: x.get_min_cds_start())
-            for i,tr in enumerate(ref_tr[:-1]):
-                    tr_dict[(tr.id,tr.source)] = tr
-                    for tr2 in ref_tr[i+1:]:
-                      if tr.is_feature_spanning(tr2):
-                        if tr.overlap_cds_with_other_transcript_cds(tr2):
-                            conflict_list.append(((tr.id,tr.source), (tr2.id,tr2.source)))
+        list_to_remove = Utils.aed_strand_filter_transcripts(transcripts)
 
-            # add last tr in dict
-            tr_dict[(ref_tr[-1].id,ref_tr[-1].source)] = ref_tr[-1]
-
-        logging.info("{} potential conflicts to resolved".format(len(conflict_list)))
-
-        list_to_remove = []
-        for cf in conflict_list:
-            #l = self._rank_tr_score([tr_dict[cf[0]],tr_dict[cf[1]]])
-            l = self._rank_tr([tr_dict[cf[0]],tr_dict[cf[1]]])
-            if l[0] not in list_to_remove:
-                list_to_remove.append(l[1])
-
-#        self.export(genes, list_to_remove)
         new_transcripts = []
-#        not_exported = 0
         for tr in transcripts:
             if tr not in list_to_remove:
                 new_transcripts.append(tr)
             else:
-#                not_exported += 1
                 transcripts_not_exported.append(tr)
 
         logging.info("{} transcripts removed due to overlapping with other CDS".format(len(transcripts_not_exported)))
