@@ -38,7 +38,10 @@ class Validate(object):
         for gene in genes:
             gff3 += gene.to_gff3()
             for tr in gene.lTranscripts:
-                gff3 += tr.to_gff3(atts=tr.dAttributes)
+                atts=tr.dAttributes
+                atts["ID"]=[tr.id]
+                atts["Parent"]=[tr.gene_id]
+                gff3 += tr.to_gff3(atts=atts)
                 for ex in sorted(tr.lExons, key=lambda x : x.start):
                     gff3 += ex.to_gff3(atts={"ID":[ex.exon_id],"Parent":[tr.id]})
                 for cds in sorted(tr.lCDS, key=lambda x : x.start):
